<?php

namespace EventPortal\Concerns;

use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Team\Models\Team;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\MorphToMany;

trait HasTeams
{
    public function teamAsOption(): Option
    {
        return Option::make(optional($this->team)->name)
            ->value(optional($this->team)->id);
    }

    public function getTeamAttribute(): ?Team
    {
        return $this->teams->first();
    }

    public function teams(): MorphToMany
    {
        return $this->morphToMany(Team::class, 'teamable')->withTimeStamps();
    }

    public function belongsToTeam(Team $team): bool
    {
        return $this->teams->pluck('id')->contains($team->id);
    }

    public function belongsToSameTeamAs(Model $model): bool
    {
        return $this->teams->pluck('id')->intersect($model->teams->pluck('id'))->isNotEmpty();
    }

    public function doesNoBelongToSameTeamAs(Model $model): bool
    {
        return $this->teams->pluck('id')->intersect($model->teams->pluck('id'))->isEmpty();
    }
}
